#include "def.h"
/* Programa que determina el tiempo promedio
   de ejecucin de un algoritmo de ordenamiento,
   como tambin su desviacin estandar.

   Argumento de entrada : largo del arreglo */

int main (int argc , char *argv[])
{
    int i , j;                  /* iteraciones */
    long n;                     /* largo del arreglo */
    int *a;                     /* puntero a direccin de memoria donde
                                   reside arreglo */
    struct timeval start_time;  /* tiempo de inicio */
    struct timeval end_time;    /* tiempo de trmino */

    float t = 0.0;              /* tiempo de ejecucin */
    float time_prom = 0.0;      /* tiempo promedio */
    float desv_std = 0.0;       /* desviacin estandar */
    float xi = 0.0;             /* variable auxiliar para promedio */
    float xi2 = 0.0;            /* variable auxiliar para desviacin estandar */

    char *pEnd;                 /* puntero a string auxiliar para convertir
                                   valor de entrada */



    /* verificacin de sintaxis */
    if (argc != 2)
    {
        printf ("Error de Sintaxis\n");
        printf ("Sintaxis : nombre_programa <largo_arreglo>\n");
        exit (1);
    }
    
    /* conversin de argumento de entrada a entero */
    n = strtol(argv[1] , &pEnd , 0);
    
    /* verificacin de argumentos */
    if (n == 0)
    {
        printf ("Error de Argumento\n");
        printf ("Argumento entrado es cero o invlido\n");
        exit (1);
    }

    /* fijando semilla para funcin random */
	srandom(time(NULL));

	/* asignacin de memoria para arreglo */
    a = (int *)malloc(n*sizeof(int));

	/* verificacin de espacio en memoria */
    if (a == NULL)
	{
		printf ("Error de memoria\n");
		exit (1);
	}

    /* ciclo de iteracin */
    for (i = 0 ; i < NUMEXP ; i++)
    {
        /* asignacin de elementos del arreglo
           aleatoriamente entre 1 y 1000 */
        for (j = 0 ; j < n ; j++)
            a[j] = 1 + (int) (1000.0*random()/RAND_MAX);

        /* obtencin del tiempo de inicio
           del algoritmo de ordenamiento */
        gettimeofday (&start_time , NULL);

        /* llamada a funcin de ordenamiento */
        ordenar (a , n);

        /* obtencin del tiempo de trmino
           del algoritmo de ordenamiento */
        gettimeofday (&end_time , NULL);

        /* obtencin tiempo de ejecucin en microsegundos */
        t = (end_time.tv_sec*1000000 + end_time.tv_usec) - (start_time.tv_sec*1000000 + start_time.tv_usec);

        /* clculo de sumatorias */
        xi += t;
        xi2 += t*t;
    }

	/* liberacin de memoria */
    free(a);

    /* clculo de tiempo promedio */
    time_prom = xi/NUMEXP;

    /* clculo de desviacin estandar */
    desv_std = sqrt( (xi2 - NUMEXP*(time_prom*time_prom)) / (NUMEXP - 1) );

    /* salida por pantalla */
    printf ("\nInformacin sobre experimentos\n");
    printf ("------------------------------\n");
    printf ("Largo del arreglo\t: %d\n" , n);
    printf ("Nmero de experimentos\t: %d\n\n" , NUMEXP);
    printf ("Estadsticas sobre tiempo de ejecucin\n");
    printf ("--------------------------------------\n");
    printf ("Tiempo promedio \t: %f [microseg]\n" , time_prom);
    printf ("Desviacin estandar \t: %f [microseg]\n\n" , desv_std);

    return 1;
}
