#include "def.h"

/*
   Funcin ordenar
   ordena arreglo de menor a mayor
   usando algoritmo Merge Sort

   A[] : arreglo desordenado
   n   : largo del arreglo
*/

int ordenar (int A[] , long n)
{
	/* asignacin de memoria para arreglo
	auxiliar referenciado por aux */
	int *aux = (int *)malloc(n*sizeof(int));

	/* verificacin de espacio en memoria */
	if (aux == NULL)
	{
		printf ("Error de memoria\n");
		exit (1);
	}

	/* llamado a Merge Sort */
	mergeSort (A , aux , 0 , n - 1);

	/* liberacin de memoria */
	free(aux);

	return 0;
}

/*
   Funcin Merge Sort

   A[]	 : arreglo desordenado
   Aux[] : arreglo auxiliar
   l  	 : primer ndice del arreglo
   r  	 : ltimo ndice del arreglo
*/
int mergeSort(int A[] , int Aux[] , int l , int r)
{
    int m;  /* ndice medio del arreglo */

    /* si arreglo tiene slo un elemento retorna */
    if (r <= l) return 0;

    /* clculo ndice medio */
    m = (l + r) / 2;

    /* llamadas recursivas */
    mergeSort(A , Aux , l , m);
    mergeSort(A , Aux , m + 1 , r);

    /* llamada a funcin mezcla */
    merge(A , Aux , l , m , r);
}

/*
   Funcin merge
   mezcla subarreglos

   A[]	 : arreglo desordenado
   Aux[] : arreglo auxiliar
   l  	 : primer ndice del arreglo
   m  	 : posicin media del arreglo
   r  	 : ltimo ndice del arreglo
*/
int merge(int A[] , int Aux[] , int l , int m , int r)
{
    int i, j, k;    /* iteraciones */

    /* copiando subarreglo inferior */
    for (i = m + 1 ; i > l ; i--)
        Aux[i-1] = A[i-1];

    /* copiando subarreglo superior */
    for (j = m ; j < r ; j++)
        Aux[r + m - j] = A[j + 1];

    /* proceso mezcla */
    for (k = l; k <= r; k++)
       if (Aux[i] < Aux[j])
          A[k] = Aux[i++];
       else
          A[k] = Aux[j--];

	return 0;
}
